import 'dart:async';
import 'package:dart_blocks/dart_blocks/models/auth.dart';
import 'package:dart_blocks/dart_blocks/nuntio_client.dart';
import 'package:dart_blocks/dart_blocks/user_analytics/user_analytics.dart';
import 'package:dart_blocks/dart_blocks/user_block_with_ui/NoConnection/no_connection.dart';
import 'package:dart_blocks/dart_blocks/user_block_with_ui/WelcomePage/welcome_page.dart';
import 'package:dart_blocks/dart_blocks/user_block_with_ui/models.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:nuntio_blocks/block_user.pb.dart';

class UserBlockWithUI extends StatefulWidget {
  UserBlockWithUI({
    Key? key,
    required this.child,
    NuntioText? nuntioText,
    NuntioColor? nuntioColor,
    this.onRegister,
    this.onLogin,
    this.textFieldBorder,
    this.logo,
    this.background,
    this.buttonHeight,
    this.buttonWidth,
    this.titleStyle,
    this.detailsStyle,
    this.createdByStyle,
    this.loginButtonTextStyle,
    this.registerButtonTextStyle,
    this.bodyTextStyle,
  }) {
    this.nuntioText = nuntioText ?? NuntioText();
    this.nuntioColor = nuntioColor ?? NuntioColor();
  }

  late final NuntioText nuntioText;
  late final NuntioColor nuntioColor;

  // general
  final Widget child;
  final Widget? logo;

  // sizes
  final double? buttonHeight;
  final double? buttonWidth;

  // style
  final BoxDecoration? background;
  final Border? textFieldBorder;

  // functions
  final Function? onRegister;
  final Function? onLogin;

  // style
  final TextStyle? detailsStyle;
  final TextStyle? titleStyle;
  final TextStyle? createdByStyle;
  final TextStyle? loginButtonTextStyle;
  final TextStyle? registerButtonTextStyle;
  final TextStyle? bodyTextStyle;

  // on authenticated go to child;
  @override
  State<UserBlockWithUI> createState() => _UserBlockWithUIState();
}

class _UserBlockWithUIState extends State<UserBlockWithUI> {
  // init
  late Future<AuthState> initializeNuntioUIFuture;
  late Config _config;

  Future<void> initializeConfig() async {
    _config = await NuntioClient.userBlock.initializeApplication();
    return;
  }

  Future<AuthState> initializeAuthStatus() async {
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.none) {
      //  no data connection
      return AuthState.noConnection;
    }
    bool isAuthenticated = await NuntioClient.userBlock.isAuthenticated();
    if (isAuthenticated) {
      return AuthState.authenticated;
    }
    return AuthState.notAuthenticated;
  }

  Future<AuthState> initializeNuntioUI() async {
    await initializeConfig();
    return await initializeAuthStatus();
  }

  _UserBlockWithUIState() {
    initializeNuntioUIFuture = initializeNuntioUI();
  }

  @override
  Widget build(BuildContext context) {
    return UserAnalytics(
      child: CupertinoPageScaffold(
        resizeToAvoidBottomInset: false,
        backgroundColor: CupertinoColors.systemBackground,
        child: FutureBuilder<AuthState>(
            future: initializeNuntioUIFuture,
            builder: (BuildContext context, AsyncSnapshot<AuthState> snapshot) {
              switch (snapshot.connectionState) {
                case ConnectionState.waiting:
                  return const Center(
                    child: CupertinoActivityIndicator(),
                  );
                case ConnectionState.done:
                  if (snapshot.data == AuthState.authenticated) {
                    return widget.child;
                  } else if (snapshot.data == null ||
                      snapshot.data == AuthState.notAuthenticated) {
                    return WelcomePage(
                      verifyCodeTitle: Text(
                        "Enter your verification code",
                        style: widget.titleStyle ??
                            Theme.of(context)
                                .textTheme
                                .titleLarge
                                ?.copyWith(color: CupertinoColors.black),
                        textAlign: TextAlign.center,
                      ),
                      arrowBackColor: widget.nuntioColor.primaryColor,
                      forgotPasswordColor: widget.nuntioColor.secondaryColor,
                      disableConnect: !_config.enableNuntioConnect,
                      disableLogin: _config.disableDefaultLogin,
                      infoColor: widget.nuntioColor.primaryColor,
                      buttonHeight: widget.buttonHeight ?? 55,
                      buttonWidth: widget.buttonWidth ?? 280,
                      disableRegistration: _config.disableDefaultSignup,
                      validatePassword: _config.validatePassword,
                      textFieldColor: CupertinoColors.white,
                      textFieldBorder: widget.textFieldBorder ??
                          Border.all(
                            color: Colors.black26,
                            width: 0.5,
                          ),
                      forgotPasswordText: Text(
                        widget.nuntioText.forgotPasswordDetails,
                        style: TextStyle(
                          color: widget.nuntioColor.primaryColor,
                        ),
                        textAlign: TextAlign.center,
                      ),
                      logo: widget.logo ??
                          Image(
                            image: AssetImage(_config.logo),
                            width: 150,
                          ),
                      background: widget.background ??
                          BoxDecoration(color: Colors.white),
                      loginButtonColor: widget.nuntioColor.primaryColor,
                      registerButtonColor: widget.nuntioColor.secondaryColor,
                      welcomeDetails: Text(
                        widget.nuntioText.welcomeDetails,
                        style: widget.detailsStyle ??
                            Theme.of(context)
                                .textTheme
                                .titleMedium
                                ?.copyWith(color: Colors.black),
                        textAlign: TextAlign.center,
                      ),
                      createdBy: Text(
                        "Created with Nuntio",
                        style: widget.createdByStyle ??
                            Theme.of(context)
                                .textTheme
                                .titleSmall
                                ?.copyWith(color: Colors.black),
                        textAlign: TextAlign.center,
                      ),
                      welcomeTitle: Text(
                        widget.nuntioText.welcomeTitle,
                        style: widget.titleStyle ??
                            Theme.of(context).textTheme.titleLarge?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      loginButtonText: Text(
                        widget.nuntioText.loginButton,
                        style: widget.loginButtonTextStyle ??
                            TextStyle(
                              color: CupertinoColors.white,
                              fontWeight: FontWeight.w400,
                            ),
                        textAlign: TextAlign.center,
                      ),
                      registerButtonText: Text(
                        widget.nuntioText.registerButton,
                        style: widget.registerButtonTextStyle ??
                            TextStyle(
                              color: CupertinoColors.black,
                              fontWeight: FontWeight.w400,
                            ),
                        textAlign: TextAlign.center,
                      ),
                      loginDetailsTitle: Text(
                        widget.nuntioText.loginDetails,
                        style: widget.detailsStyle ??
                            Theme.of(context).textTheme.titleMedium?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      onLogin: () => {
                        if (widget.onLogin != null) {widget.onLogin!()},
                        setState(() {
                          initializeNuntioUIFuture =
                              initializeNuntioUI().catchError((onError) {
                            print("Something went wrong" + onError.toString());
                          }).then((isAuthenticated) {
                            print("User is authenticated:" +
                                isAuthenticated.toString());
                            if (isAuthenticated == AuthState.authenticated) {
                              Navigator.pop(context);
                            }
                            return isAuthenticated;
                          });
                        }),
                      },
                      loginTitle: Text(
                        widget.nuntioText.loginTitle,
                        style: widget.titleStyle ??
                            TextStyle(
                              color: CupertinoColors.black,
                            ),
                        textAlign: TextAlign.center,
                      ),
                      registerTitle: Text(
                        widget.nuntioText.registerTitle,
                        style: widget.titleStyle ??
                            TextStyle(
                              color: CupertinoColors.black,
                            ),
                        textAlign: TextAlign.center,
                      ),
                      passwordLoginHint: widget.nuntioText.passwordHint,
                      emailLoginHint: widget.nuntioText.identifierHint,
                      onRegister: () => {
                        if (widget.onRegister != null) {widget.onRegister!()},
                        setState(() {
                          initializeNuntioUIFuture =
                              initializeNuntioUI().then((isAuthenticated) {
                            if (isAuthenticated == AuthState.authenticated) {
                              Navigator.pop(context);
                            }
                            return isAuthenticated;
                          });
                        }),
                      },
                      registerDetails: Text(
                        widget.nuntioText.registerDetails,
                        style: widget.detailsStyle ??
                            Theme.of(context).textTheme.titleMedium?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      passwordRegisterHint: widget.nuntioText.passwordHint,
                      repeatPasswordRegisterHint:
                          widget.nuntioText.repeatPasswordHint,
                      emailRegisterHint: widget.nuntioText.identifierHint,
                      missingPasswordTitle:
                          widget.nuntioText.missingPasswordTitle,
                      missingPasswordDetails:
                          widget.nuntioText.missingPasswordDescription,
                      missingEmailTitle:
                          widget.nuntioText.missingIdentifierTitle,
                      missingEmailDetails:
                          widget.nuntioText.missingIdentifierDescription,
                      invalidTitle: widget.nuntioText.errorTitle,
                      invalidDetails: widget.nuntioText.errorDescription,
                      passwordDoNotMatchTitle:
                          widget.nuntioText.passwordDoNotMatchTitle,
                      passwordDoNotMatchDetails:
                          widget.nuntioText.passwordDoNotMatchDescription,
                      errorColor: widget.nuntioColor.secondaryColor,
                      successColor: widget.nuntioColor.secondaryColor,
                      containsEightCharactersText: Text(
                        widget.nuntioText.passwordContainsEightCharsHint,
                        style: widget.bodyTextStyle ??
                            Theme.of(context).textTheme.bodyText1?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                      ),
                      containsSpecialText: Text(
                        widget.nuntioText.passwordContainsSpecialHint,
                        style: widget.bodyTextStyle ??
                            Theme.of(context).textTheme.bodyText1?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                      ),
                      containsNumberText: Text(
                        widget.nuntioText.passwordContainsNumberHint,
                        style: widget.bodyTextStyle ??
                            Theme.of(context).textTheme.bodyText1?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                      ),
                      passwordMatchText: Text(
                        widget.nuntioText.passwordsMustMatchHint,
                        style: widget.bodyTextStyle ??
                            Theme.of(context).textTheme.bodyText1?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                      ),
                    );
                  } else {
                    print(snapshot.data);
                    return NoConnection(
                      createdBy: Text(
                        "Created with Nuntio",
                        style: widget.createdByStyle ??
                            Theme.of(context).textTheme.titleSmall?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      title: Text(
                        widget.nuntioText.noWifiTitle,
                        style: widget.titleStyle ??
                            Theme.of(context).textTheme.titleLarge?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      details: Text(
                        widget.nuntioText.noWifiDescription,
                        style: widget.detailsStyle ??
                            Theme.of(context).textTheme.titleMedium?.copyWith(
                                  color: CupertinoColors.black,
                                ),
                        textAlign: TextAlign.center,
                      ),
                      logo: widget.logo ??
                          Text(
                            "Move fast. Stay in control.",
                            style: widget.titleStyle ??
                                Theme.of(context)
                                    .textTheme
                                    .titleLarge
                                    ?.copyWith(
                                      color: CupertinoColors.black,
                                    ),
                            textAlign: TextAlign.center,
                          ),
                      background: widget.background ??
                          BoxDecoration(color: CupertinoColors.white),
                      primaryColor: widget.nuntioColor.primaryColor,
                    );
                  }
                default:
                  return Text("Error");
              }
            }),
      ),
    );
  }
}
